@@ -16,7 +16,6 @@ import sys
 import textwrap
 
 import aiohttp
-import numpy as np
 import pandas as pd
 
 try:
@@ -71,10 +70,20 @@ class SiPM:
 
 ##############################################################################
 # Methods of identifying good/bad SiPMs
+#
+# sipm_status_full_check
+#     fetches individual SiPMs sequentially from table sipm_test
+#
+# sipm_status_full_check_async
+#     fetches individual SiPMs asynchronously from table sipm_test
+#
+# sipm_status_whole_table
+#     fetches the entire sipm_test table
+#
 ##############################################################################
 
 
-def identify_sipm_status(dbi, classification, wafer_pid, sequential, no_b_grade, qcid=None, nounusablesipmcheck=False):
+def identify_sipm_status(dbi, wafer_pid, sequential, no_b_grade, qcid=None, nounusablesipmcheck=False):
     """
     For the given wafer, obtain two sets of wafer (column, row) pairs, for
     good/bad SiPMs respectively. Chooses between two methods of accomplishing
@@ -96,8 +105,6 @@ def identify_sipm_status(dbi, classification, wafer_pid, sequential, no_b_grade,
         dbi : ds20kdb.interface.Database
             Instance of the Database interface class; allows communication
             with the database.
-        full : bool
-            Perform a comprehensive check of SiPM status.
         wafer_pid : int
         sequential : bool
         no_b_grade : bool
@@ -114,13 +121,14 @@ def identify_sipm_status(dbi, classification, wafer_pid, sequential, no_b_grade,
 
     dfr = response.data
 
+    if dfr is None:
+        print('WARNING: no response from database')
+        return None
+
     if dfr.empty:
         print('WARNING: empty response')
         return None
 
-    if classification:
-        return sipm_status_by_classification(dbi, dfr)
-
     if nounusablesipmcheck:
         unusable_sipm_ids = set()
     else:
@@ -132,89 +140,6 @@ def identify_sipm_status(dbi, classification, wafer_pid, sequential, no_b_grade,
     return sipm_status_whole_table(dbi, dfr, unusable_sipm_ids, no_b_grade, qcid)
 
 
-def sipm_status_by_classification(dbi, dfr):
-    """
-    For the given wafer, obtain two sets of wafer (column, row) pairs, for
-    good/bad SiPMs respectively.
-
-    Uses just sipm_test.classification for the evaluation.
-
-    Note that this check as currently implemented is overly pessimistic. If a
-    SiPM ID has ever been assigned a bad classification, it will show as bad
-    here. In Q4 2023, <1% of SiPM IDs had multiple entries, and a later
-    corrected entry may contain a good result.
-
-    --------------------------------------------------------------------------
-    args
-        dbi : ds20kdb.interface.Database
-            Instance of the Database interface class; allows communication
-            with the database.
-        dfr : Pandas DataFrame
-    --------------------------------------------------------------------------
-    returns (set, set, set)
-    --------------------------------------------------------------------------
-    """
-    print('Obtaining SiPMs with bad classification(s)')
-
-    bad_sipm_ids = set(dbi.get('sipm_test', classification='bad').data.sipm_id)
-    wafer_map_bad = {
-        (col, row)
-        for sipm_pid, col, row in zip(dfr.sipm_pid, dfr.column, dfr.row)
-        if sipm_pid in bad_sipm_ids
-    }
-
-    all_locations = set(interface.wafer_map_valid_locations())
-    wafer_map_good = all_locations.difference(wafer_map_bad)
-
-    wafer_map_b_grade = {}
-    return wafer_map_good, wafer_map_b_grade, wafer_map_bad
-
-
-def identify_b_grade_sipms(dbi, dfr, good_sipm_ids, qcid, sipm_test):
-    """
-    SK's additional checks based on device performance.
-
-    Here we are returning generally good SiPMs. These passed electrical and
-    visual inspection at LFoundry, and passed electrical tests at NOA.
-    However, their performance is at the margins of acceptable performance.
-
-    These SiPMs will not be discarded, they will be retained and handled as
-    production silicon, except that they will not be used in production
-    vTiles. They may be used towards the end of production if required.
-
-    --------------------------------------------------------------------------
-    args
-        dbi : ds20kdb.interface.Database
-            Instance of the Database interface class; allows communication
-            with the database.
-        dfr : Pandas DataFrame
-        good_sipm_ids : set of int
-        qcid : int
-        sipm_test : pandas DataFrame
-    --------------------------------------------------------------------------
-    returns set of tuple
-        e.g. {(int, int), ...} or None
-        notionally these are B-grade SiPMs
-    --------------------------------------------------------------------------
-    """
-    try:
-        sipm = SiPMCheck(dbi, qcid, sipm_test)
-    except (LookupError, TimeoutError):
-        return None
-
-    b_grade_sipm_ids = {
-        sipm_id
-        for sipm_id in good_sipm_ids
-        if not sipm.production_standard(sipm_id)
-    }
-
-    return {
-        (col, row)
-        for sipm_pid, col, row in zip(dfr.sipm_pid, dfr.column, dfr.row)
-        if sipm_pid in b_grade_sipm_ids
-    }
-
-
 def wafer_map_sets(dbi, dfr, good_sipm_ids, no_b_grade, qcid, sipm_test=None):
     """
     Generate the sets of locations necessary to generate the wafer map.
@@ -240,15 +165,7 @@ def wafer_map_sets(dbi, dfr, good_sipm_ids, no_b_grade, qcid, sipm_test=None):
         if sipm_pid in good_sipm_ids
     }
 
-    # Add SK's additional checks for device performance
-    if no_b_grade:
-        wafer_map_b_grade = set()
-    else:
-        wafer_map_b_grade = identify_b_grade_sipms(dbi, dfr, good_sipm_ids, qcid, sipm_test)
-        if wafer_map_b_grade is None:
-            wafer_map_b_grade = set()
-            print('ERROR: could not read status of b-grade SiPMs')
-
+    wafer_map_b_grade = set()
     wafer_map_good -= wafer_map_b_grade
 
     all_locations = set(interface.wafer_map_valid_locations())
@@ -274,6 +191,7 @@ def sipm_status_full_check(dbi, dfr, unusable_sipm_ids, no_b_grade, qcid):
             Instance of the Database interface class; allows communication
             with the database.
         dfr : Pandas DataFrame
+            SiPMs for a given wafer ID
         unusable_sipm_ids : set of int
         no_b_grade : bool
         qcid : int
@@ -308,6 +226,8 @@ def sipm_status_full_check(dbi, dfr, unusable_sipm_ids, no_b_grade, qcid):
             if classification == 'good' and quality_flag == 0:
                 good_sipm_ids.add(sipm_id)
 
+    # good_sipm_ids will contain all SiPMs with satisfactory electrical test
+    # results that also passed NOA's visual inspection regime.
     good_sipm_ids -= unusable_sipm_ids
 
     return wafer_map_sets(dbi, dfr, good_sipm_ids, no_b_grade, qcid)
@@ -325,12 +245,15 @@ def sipm_status_whole_table(dbi, dfr, unusable_sipm_ids, no_b_grade, qcid):
     table here, so it's more efficient just to load the whole table here and
     pass it to SiPMCheck.
 
+    Note that the time to load the table >> search/processing time.
+
     --------------------------------------------------------------------------
     args
         dbi : ds20kdb.interface.Database
             Instance of the Database interface class; allows communication
             with the database.
         dfr : Pandas DataFrame
+            SiPMs for a given wafer ID
         unusable_sipm_ids : set of int
         no_b_grade : bool
         qcid : int
@@ -370,6 +293,8 @@ def sipm_status_whole_table(dbi, dfr, unusable_sipm_ids, no_b_grade, qcid):
             if classification == 'good' and quality_flag == 0:
                 good_sipm_ids.add(sipm_id)
 
+    # good_sipm_ids will contain all SiPMs with satisfactory electrical test
+    # results that also passed NOA's visual inspection regime.
     good_sipm_ids -= unusable_sipm_ids
 
     return wafer_map_sets(dbi, dfr, good_sipm_ids, no_b_grade, qcid, sipm_test)
@@ -522,171 +447,17 @@ def sipm_status_full_check_async(dbi, dfr, unusable_sipm_ids, no_b_grade, qcid):
             if classification == 'good' and quality_flag == 0:
                 good_sipm_ids.add(sipm_id)
 
+    # good_sipm_ids will contain all SiPMs with satisfactory electrical test
+    # results that also passed NOA's visual inspection regime.
     good_sipm_ids -= unusable_sipm_ids
 
     return wafer_map_sets(dbi, dfr, good_sipm_ids, no_b_grade, qcid)
 
 
-##############################################################################
-# Code courtesy of Seraphim Koulosousas
-##############################################################################
-
-
-class SiPMCheck:
-    """
-    Class of functions to check a sipm test values against the NOA test
-    distributions.
-    """
-    def __init__(self, dbi, qcid=None, sipm_test=None):
-        """
-        loading the database separately to allow the following functions to be
-        able to loop over multiple sipms. If more tables are needed can
-        thread the loading and save in a list
-        """
-        self.dbi = dbi
-
-        # no need to load the table if we already have it in memory
-        if sipm_test is None:
-            response = self.dbi.get('sipm_test')
-            if response.network_timeout:
-                raise TimeoutError
-            if response.data is None:
-                raise LookupError
-            self.df_sipm_qc = response.data
-        else:
-            self.df_sipm_qc = sipm_test
-
-        if qcid is None:
-            # Get Latest QA/QC criteria
-            self.current_sipm_qc = self.df_sipm_qc['sipm_qc_id'].max()
-        else:
-            self.current_sipm_qc = qcid
-
-        # Get NOA test results for sipms passing the latest quality qc cuts
-        self.df_sipm_good = self.df_sipm_qc.loc[
-            (self.df_sipm_qc['quality_flag'] == 0)
-            & (self.df_sipm_qc['sipm_qc_id'] == self.current_sipm_qc)
-        ]
-
-    @staticmethod
-    def cdf(counts, bins, pct):
-        """
-        Simple function to make a CDF of the NOA Distributions. They generally
-        don't seem to follow any standard analytical function, so this is a
-        bit of a workaround.
-
-        This function looks for the x value intersect where the
-        sum == 1 - pct. (i.e P(X<x) == 1-pct).
-
-        ----------------------------------------------------------------------
-        args
-            counts : numpy.ndarray
-            bins : numpy.ndarray
-            pct : float
-        ----------------------------------------------------------------------
-        returns list or None
-        ----------------------------------------------------------------------
-        """
-        try:
-            binsize = bins[1] - bins[0]
-        except IndexError:
-            return None
-
-        acc = 0
-        for index, _bin in enumerate(bins):
-            if acc > pct:
-                return bins[index]
-
-            acc += counts[index] * binsize
-
-        return None
-
-    def stat(self, required_test, sipm_id):
-        """
-        Function that checks the sipm id against the NOA Test results.
-
-        ----------------------------------------------------------------------
-        args
-            sipm_id : int
-                the id of the sipm you are checking against Noa's database
-            required_test : tuple (string, float)
-                e.g. ('i_at_35v', 0.01)
-                    statistic : string
-                            -> chi2_shape
-                            -> i_at_20v
-                            -> rq
-                            -> i_at_35v
-                    proportion_cut : float
-                        proportion of the distribution to be cut
-                        (i.e 0.01 == 1%)
-        ----------------------------------------------------------------------
-        returns : bool or None
-        ----------------------------------------------------------------------
-        """
-        statistic, proportion_cut = required_test
-
-        # Get SiPM Data
-        sipm_data = self.df_sipm_qc.loc[self.df_sipm_qc['sipm_id'] == sipm_id]
-
-        # Convert dataframe to numpy array
-        statistic_array = self.df_sipm_good[statistic].to_numpy()
-        counts, bins = np.histogram(statistic_array, bins=200, density=True)
-
-        # default: 100 - 1%
-        pct = 1 - proportion_cut
-
-        # get the xValue that corresponds to the specified Proportion Cut
-        xval = self.cdf(counts, bins, pct)
-
-        try:
-            sipm_stat_value = sipm_data[statistic].values[0]
-        except IndexError:
-            print(f'[ERROR]: no data for sipm id: {sipm_id}')
-            return None
-
-        # True: SiPM good
-        try:
-            return sipm_stat_value < xval
-        except TypeError:
-            return None
-
-    def production_standard(self, sipm_id):
-        """
-        Run all the required assessments on the SiPM. The statistic specified
-        in the 'tests' variable is the desired field from table 'sipm_test'.
-
-        dbi.describe('sipm_test').data
-        [
-            'id', 'timestamp', 'institute_id', 'operator', 'sipm_id',
-            'vbd', 'rq', 'i_at_20v', 'i_at_35v', 'iv_rev', 'iv_fwd',
-            'chi2_shape', 'sipm_qc_id', 'group', 'temperature',
-            'classification', 'quality_flag', 'comment', 'i_at_25v'
-        ]
-
-        ----------------------------------------------------------------------
-        args
-            sipm_id : int
-                the id of the sipm you are checking against Noa's database
-        ----------------------------------------------------------------------
-        returns : bool
-        ----------------------------------------------------------------------
-        """
-        # [(statistic_test, cut_ratio), ...]
-        # cut_ratio: proportion of the distribution to be cut (i.e 0.01 == 1%)
-        tests = [
-            ('chi2_shape', 0.01),
-            ('i_at_20v', 0.01),
-            # quenching resistance
-            ('rq', 0.01),
-            ('i_at_35v', 0.01),
-        ]
-        stat_test = functools.partial(self.stat, sipm_id=sipm_id)
-
-        return all(map(stat_test, tests))
-
-
 ##############################################################################
 # python 3.6 compatibility
+#
+# See https://docs.python.org/3.6/library/itertools.html#itertools-recipes
 ##############################################################################
 
 
@@ -709,6 +480,20 @@ def grouper(iterable, n, fillvalue=None):
     return itertools.zip_longest(*args, fillvalue=fillvalue)
 
 
+###############################################################################
+# Python 3.8 compatibility
+#
+# See: https://docs.python.org/3.8/library/itertools.html#itertools-recipes
+###############################################################################
+
+
+def pairwise(iterable):
+    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
+    a, b = itertools.tee(iterable)
+    next(b, None)
+    return zip(a, b)
+
+
 ##############################################################################
 # tray file i/o
 ##############################################################################
@@ -1267,7 +1052,12 @@ def create_tray_files(var, lot, wafer_number, write):
     return lines_for_all_files
 
 
-def box_message(message, text_width):
+##############################################################################
+# pretty print
+##############################################################################
+
+
+def box_message(message, text_width=80):
     """
     Print message in a box, fitted to the console size, e.g.
 
@@ -1298,3 +1088,243 @@ def box_message(message, text_width):
     ]
 
     return '\n'.join([header, *filling, footer]) + '\n\n'
+
+
+def _consecutive_integer_sequences(nseq):
+    """
+    Generate pairs indicating the start and end of each consequtive sequence
+    of integers.
+
+    ---------------------------------------------------------------------------
+    args
+        nseq : list of int
+            e.g. [2, 4, 5, 6, 8, 9, 18, 19, 20, 21, 22, 23]
+    ---------------------------------------------------------------------------
+    yields : generator of (int, int) tuples
+        e.g. (2, 2), (4, 6), (8, 9), (18, 23)
+    ---------------------------------------------------------------------------
+    """
+    if not nseq:
+        return
+
+    snseq = sorted(nseq)
+    start = snseq[0]
+    second = snseq[-1]
+
+    for first, second in pairwise(snseq):
+        if second != first + 1:
+            # end of consecutive sequence reached
+            yield start, first
+            start = second
+
+    # final tuple
+    yield start, second
+
+
+def _pair_to_str(first, second):
+    """
+    Transform a pair of numbers into a shortened descriptive string. It is
+    expected that the numbers will be supplied sorted from low to high.
+
+    '..' is used in preference to '-' so this may be used with negative
+    integers, though this was not the original design intent.
+
+    ---------------------------------------------------------------------------
+    args
+        pair : (int, int)
+            e.g. (2, 2) or (4, 6)
+    ---------------------------------------------------------------------------
+    returns : string
+        e.g. '2' or '4..6'
+    ---------------------------------------------------------------------------
+    """
+    return f'{first}' if first == second else f'{first}..{second}'
+
+
+def compact_integer_sequence(integer_sequence):
+    """
+    Transform a list of ints to a form with compacted consecutive sequences
+    which makes it easier to read.
+
+    ---------------------------------------------------------------------------
+    args
+        integer_sequence : iterable of int
+            e.g. [2, 4, 5, 6, 8, 9, 18, 19, 20, 21, 22, 23]
+    ---------------------------------------------------------------------------
+    returns : string
+        e.g. '2, 4..6, 8..9, 18..23'
+    ---------------------------------------------------------------------------
+    """
+    return ', '.join(
+        itertools.starmap(
+            _pair_to_str,
+            _consecutive_integer_sequences(integer_sequence)
+        )
+    )
+
+
+def _expand_int_range(irange):
+    """
+    Expand a integer range encoded as a string into the matching list of ints.
+
+    ---------------------------------------------------------------------------
+    args
+        irange : string
+            e.g. '2' or '4..6'
+    ---------------------------------------------------------------------------
+    returns : list of int (or [] in the case of an unrecognised separator)
+        e.g. [2] or [4, 5, 6]
+    ---------------------------------------------------------------------------
+    """
+    try:
+        value = int(irange)
+    except ValueError:
+        # probably a sequence
+        try:
+            start, end = map(int, irange.split('..'))
+        except ValueError:
+            return []
+
+        return list(range(start, end + 1))
+
+    return [value]
+
+
+def expand_integer_sequence(compact):
+    """
+    Expand compact integer sequence as created by compact_integer_sequence()
+    into a sorted list.
+
+    ---------------------------------------------------------------------------
+    args
+        compact : string
+            e.g. '2, 4..6, 8..9, 18..23'
+    ---------------------------------------------------------------------------
+    returns : list of int
+        e.g. [2, 4, 5, 6, 8, 9, 18, 19, 20, 21, 22, 23]
+    ---------------------------------------------------------------------------
+    """
+    return sorted(
+        itertools.chain.from_iterable(
+            map(_expand_int_range, compact.split(','))
+        )
+    )
+
+
+##############################################################################
+# utilities
+##############################################################################
+
+
+def si_prefix(value, dec_places=3, compact=True):
+    """
+    Provide an approximation of the given number in engineering
+    representation, to the given number of decimal places, with the SI
+    (Systeme Internationale) unit prefix appended.
+
+    Note that for most purposes you can also use this:
+
+    from matplotlib.ticker import EngFormatter
+    fmt = EngFormatter(<ARGUMENTS>)
+    fmt.format_data(<VALUE>)
+
+    https://matplotlib.org/stable/api/ticker_api.html
+        #matplotlib.ticker.EngFormatter
+
+    --------------------------------------------------------------------------
+    args
+        value : string, float or int
+            numeric value
+        dec_places : int
+            number of decimal places to display
+        compact : bool
+            if True remove trailing zeros from number, if the remainder is a
+            whole number, remove the trailing decimal point
+            e.g.
+            103.100 -> 103.1
+             10.000 ->  10
+    --------------------------------------------------------------------------
+    returns : string or None
+        value with SI unit prefix
+    --------------------------------------------------------------------------
+    """
+    # make sure the number is in scientific notation
+    # then separate value and exponent
+    try:
+        significand, exponent = f'{float(value):e}'.lower().split('e')
+    except TypeError:
+        return None
+
+    significand = float(significand)
+    exponent = int(exponent)
+
+    # align with 10**3 boundaries
+    while exponent % 3 != 0:
+        exponent -= 1
+        significand *= 10
+
+    if -24 <= exponent <= 24:
+        # derive SI unit prefix
+        if exponent == 0:
+            prefix = ''
+        else:
+            prefix = 'yzafpnum kMGTPEZY'[8 + exponent // 3]
+
+        # remove trailing zeroes
+        # if the number is a whole number, remove the trailing decimal point as well
+        significand = f'{significand:.{dec_places}f}'
+        if compact:
+            # avoid rstrip('0.') to ensure '0.0' doesn't become ''
+            significand = significand.rstrip('0').rstrip('.')
+    else:
+        # handle the case where the supplied value is too large or too small
+        prefix = ''
+        significand = float(value)
+
+    return f'{significand}{prefix}'
+
+
+def sipm_origin_to_tuple(origin):
+    """
+    Convert a SiPM's original wafer location from dict form to tuple.
+
+    --------------------------------------------------------------------------
+    args
+        origin : dict
+            e.g. {'lot': 9262109, 'wafer_number': 3, 'column': 9, 'row': 18}
+    --------------------------------------------------------------------------
+    returns : tuple (int, int, int, int) or None
+            e.g. (9, 18, 9262109, 3)
+    --------------------------------------------------------------------------
+    """
+    try:
+        return (
+            origin['column'],
+            origin['row'],
+            origin['lot'],
+            origin['wafer_number'],
+        )
+    except (KeyError, TypeError):
+        return None
+
+
+def sipm_origin_to_dict(origin):
+    """
+    Convert a SiPM's original wafer location from tuple form to dict.
+
+    --------------------------------------------------------------------------
+    args
+        origin : tuple (int, int, int, int)
+            e.g. (9, 18, 9262109, 3)
+    --------------------------------------------------------------------------
+    returns : dict or None
+        e.g. {'lot': 9262109, 'wafer_number': 3, 'column': 9, 'row': 18}
+    --------------------------------------------------------------------------
+    """
+    try:
+        return {
+            'lot': origin[2], 'wafer_number': origin[3],
+            'column': origin[0], 'row': origin[1]
+        }
+    except (IndexError, TypeError):
+        return None
