@@ -7,7 +7,6 @@ import collections
 import contextlib
 from enum import IntEnum
 import functools
-import glob
 import itertools
 import json
 import os
@@ -123,9 +122,6 @@ class Gui:
         )
 
         self.tray_file_directory = self.home_directory
-        self.yellow_tray_directory = os.path.join(
-            self.tray_file_directory, 'yellow'
-        )
 
         # build GUI
 
@@ -135,9 +131,6 @@ class Gui:
         self.liverpool_wafers = None
         self.wafer_table_dataframe = self.database_alive()
 
-        self.disable_yellow_sipm_separation = tkinter.BooleanVar()
-        self.disable_yellow_sipm_separation.set(True)
-
         # Only load wafers that are located at the University of Liverpool.
         self.lots = self.local_wafers()
 
@@ -148,35 +141,10 @@ class Gui:
 
         self.load_defaults()
 
-        self.report_yellow_state(initial=True)
-
     ##########################################################################
     # utilities
     ##########################################################################
 
-    def report_yellow_state(self, initial=False):
-        """
-        Report to the user whether tray files for yellow SiPMs (those at the
-        1% margin of acceptable performance) will be generated.
-
-        ----------------------------------------------------------------------
-        args
-            initial : boolean
-                flag to indicate if this call is the one made at application
-                startup. If so, display some additional information.
-        ----------------------------------------------------------------------
-        returns : none
-            state of console will change
-        ----------------------------------------------------------------------
-        """
-        status = 'DISABLED' if self.disable_yellow_sipm_separation.get() else 'ENABLED'
-        message = f'Generate tray files for yellow SiPMs: {status}'
-
-        if initial:
-            message = f'\n{message}\nThis setting may be changed from the Tools menu.'
-
-        self.print_to_console(message)
-
     def database_alive(self):
         """
         Load something relatively small to check connectivity, and also gives
@@ -196,34 +164,6 @@ class Gui:
 
         return response.data
 
-    def max_yellow_file_number(self):
-        """
-        Read filenames from the directory containing yellow SiPM tray files
-        and obtain the highest number contained in all the filenames.
-
-        Expected filename format: yellow_NNNN.txt, e.g. yellow_0123.txt
-
-        ----------------------------------------------------------------------
-        args : none
-        ----------------------------------------------------------------------
-        returns : int
-        ----------------------------------------------------------------------
-        """
-        # get existing files
-        yellow_tray_files = glob.glob(
-            os.path.join(self.yellow_tray_directory, '*.txt')
-        )
-        if not yellow_tray_files:
-            return 0
-
-        numbers = []
-        for ytf in yellow_tray_files:
-            fn_no_ext = os.path.splitext(os.path.basename(ytf))[0]
-            with contextlib.suppress(ValueError):
-                numbers.append(int(fn_no_ext.rsplit('_')[-1]))
-
-        return max(numbers)
-
     def local_wafers(self):
         """
         Obtain a list of wafer lot numbers for wafers present at the
@@ -377,9 +317,6 @@ class Gui:
         ----------------------------------------------------------------------
         """
         self.tray_file_directory = directory
-        self.yellow_tray_directory = os.path.join(
-            self.tray_file_directory, 'yellow'
-        )
 
     ##########################################################################
     # build GUI
@@ -405,12 +342,6 @@ class Gui:
             label='Set default tray file directory',
             command=self.set_default_tray_file_directory,
         )
-        tool_menu.add_checkbutton(
-            label="Disable \"yellow\" SiPM separation",
-            onvalue=True, offvalue=False,
-            variable=self.disable_yellow_sipm_separation,
-            command=self.report_yellow_state
-        )
 
         help_menu.add_command(
             label='About',
@@ -719,14 +650,7 @@ class Gui:
         self.frame_tab[Tab.GREEN].grid(column=0, row=0)
         self.frame_tab[Tab.GREEN].grid_propagate(False)
 
-        self.frame_tab[Tab.YELLOW] = ttk.Frame(
-            notebook, width=self.left_frame_width - 8, height=290
-        )
-        self.frame_tab[Tab.YELLOW].grid(column=0, row=0)
-        self.frame_tab[Tab.YELLOW].grid_propagate(False)
-
         notebook.add(self.frame_tab[Tab.GREEN], text='GREEN (production standard)')
-        notebook.add(self.frame_tab[Tab.YELLOW], text='YELLOW (1% cut victims)')
 
         # populate tabs
         self.button['select_green'] = ttk.Button(
@@ -742,22 +666,9 @@ class Gui:
         )
         self.button['clear_green'].grid(column=1, row=0, sticky=tkinter.W)
 
-        self.button['select_yellow'] = ttk.Button(
-            self.frame_tab[Tab.YELLOW],
-            text='select file(s)',
-            command=lambda: self.select_tray_filenames(Tab.YELLOW),
-        )
-        self.button['select_yellow'].grid(column=0, row=0, padx=8, pady=8, sticky=tkinter.W)
-        self.button['clear_yellow'] = ttk.Button(
-            self.frame_tab[Tab.YELLOW],
-            text='clear file list',
-            command=lambda: self.clear_filenames(Tab.YELLOW),
-        )
-        self.button['clear_yellow'].grid(column=1, row=0, sticky=tkinter.W)
-
         # add blank labels to each tab
         for tab, row in itertools.product(
-            [Tab.GREEN, Tab.YELLOW], range(1, 1 + self.max_filenames_per_tab)
+            [Tab.GREEN], range(1, 1 + self.max_filenames_per_tab)
         ):
             a = tkinter.Label(self.frame_tab[tab], text='')
             a.grid(column=0, row=row, columnspan=2, sticky=tkinter.W)
@@ -1077,12 +988,11 @@ class Gui:
         self.print_to_console('Retrieving wafer map details, please wait...')
 
         try:
-            wafer_map_green, wafer_map_yellow, wafer_map_red = common.identify_sipm_status(
+            wafer_map_green, _wafer_map_yellow, wafer_map_red = common.identify_sipm_status(
                 self.dbi,
-                classification=False,
                 wafer_pid=wafer_pid,
                 sequential=False,
-                no_b_grade=self.disable_yellow_sipm_separation.get(),
+                no_b_grade=True,
             )
         except TypeError:
             self.print_to_console('Identifying SiPM status failed.')
@@ -1090,21 +1000,17 @@ class Gui:
 
         # these variables may be altered during generation of tray files
         wmg = wafer_map_green.copy()
-        wmy = wafer_map_yellow.copy()
         wmr = wafer_map_red.copy()
 
         # sort: highest column first, then lowest row first
         wafer_map_green = sorted(wafer_map_green, key=lambda x: (-x[1], x[0]))
-        wafer_map_yellow = sorted(wafer_map_yellow, key=lambda x: (-x[1], x[0]))
 
         self.print_to_console(
             f'green {len(wafer_map_green)}, '
-            f'yellow {len(wafer_map_yellow)}, '
             f'red {len(wafer_map_red)}'
         )
 
         existing_green = common.evaluate_tray_files(self.filenames[Tab.GREEN])
-        existing_yellow = common.evaluate_tray_files(self.filenames[Tab.YELLOW])
 
         ##########################################################################
         # fill existing tray files with SiPMs
@@ -1112,10 +1018,9 @@ class Gui:
 
         tasks = [
             ('GREEN', existing_green, wafer_map_green),
-            ('YELLOW', existing_yellow, wafer_map_yellow),
         ]
 
-        if any([existing_green, existing_yellow]):
+        if any([existing_green]):
             self.print_to_console(
                 f'\n{"#" * 58}\n# Filling existing files\n{"#" * 58}'
             )
@@ -1150,18 +1055,9 @@ class Gui:
             # will be written to this location in all cases.
             Path(path).mkdir(parents=True, exist_ok=True)
 
-            if wafer_map_yellow:
-                Path(self.yellow_tray_directory).mkdir(parents=True, exist_ok=True)
-
         # sipm type, wafer map, directory, file number offset
         tasks = [
             ('green', wafer_map_green, path, 0),
-            (
-                'yellow',
-                wafer_map_yellow,
-                self.yellow_tray_directory,
-                self.max_yellow_file_number()
-            ),
         ]
 
         status_lines = map(
@@ -1195,12 +1091,6 @@ class Gui:
                 'sipm_colour': 'darkred',
                 'text_colour': 'lightgrey',
             },
-            {
-                'name': 'bad_noa-test-stats',
-                'locations': wmy,
-                'sipm_colour': 'darkgoldenrod',
-                'text_colour': 'black',
-            },
         ]
 
         restore_cwd = os.getcwd()
