@@ -58,6 +58,7 @@ import time
 import types
 
 import paramiko
+import PIL
 from PIL import Image
 
 from ds20kdb import interface
@@ -73,7 +74,7 @@ class Connect:
     Create a persistent SSH connection for use by a context manager.
     """
     # Nodes running CentOS 7 (EOL 2024 06 30) retired
-    # Use this node with AlmaLinux release 9.3 (Shamrock Pampas Cat)
+    # Use this node with AlmaLinux 9.x (can also use linappserv{1,2})
     remote_filestore = 'linappserv6.pp.rhul.ac.uk'
 
     def __init__(self, args):
@@ -214,12 +215,12 @@ class Component:
 
         compress_level=6 is the default, compress_level=4 represents the best
         compromise between file size and compression speed for the large
-        (~115MB) uncompressed TIF scans used here.
+        (~115MiB) uncompressed TIF scans used here.
 
         Time is given relative to compress_level=9 (105.25s) for a
         representative scan
 
-        compress_level    time    file size (MB)
+        compress_level    time    file size (MiB)
          0                0.02     114
          1                0.03      49
          2                0.04      48
@@ -261,7 +262,7 @@ class Component:
     @staticmethod
     def report_upload_size(scan_for_upload):
         """
-        Print file size in MB.
+        Print file size in MiB.
 
         ----------------------------------------------------------------------
         args
@@ -274,7 +275,7 @@ class Component:
         image_size_bytes = scan_for_upload.tell()
         scan_for_upload.seek(0)
 
-        print(f'Uploading ~{image_size_bytes // (1024 * 1024)}MB...')
+        print(f'Uploading ~{image_size_bytes // (1024 * 1024)}MiB...')
 
     def upload_image(self):
         """
@@ -312,7 +313,7 @@ class Component:
             url = f'https://www.pp.rhul.ac.uk/DarkSide/scanner/{self.institute}/{filename}'
 
             # This temporarily loads the entire image into memory. Scanned
-            # images should be a maximum of 60MB in size (PNG), so no
+            # images should be a maximum of 60MiB in size (PNG), so no
             # problems are expected with memory usage. SHA256 is used instead
             # of blake2b simply because sha256sum is ubiquitous and available
             # at the far end from the command line.
@@ -345,6 +346,9 @@ class Component:
                 except OSError:
                     # OSError: size mismatch in put!
                     pass
+                except EOFError:
+                    # remote host closed connection mid-transfer
+                    pass
                 else:
                     sftp.utime(remote_file_path, (local_file_mtime, local_file_mtime))
 
@@ -523,7 +527,19 @@ class Component:
         print("Table:\t", self.component + '_test')
         print(self.table)
 
-        post_successful = self.dbi.post_measurement(self.table, self.component)
+        # Rare transient outages have been observed when accessing the
+        # production database (which may be attributed to remote or local
+        # issues). Seek to avoid a time-consuming manual fix for the file
+        # being uploaded to the filestore without a matching database entry.
+        attempts = 3
+        for attempt in range(attempts, 0, -1):
+            post_successful = self.dbi.post_measurement(self.table, self.component)
+            if post_successful:
+                break
+            if attempt > 1:
+                time.sleep(10)
+        else:
+            print(f'Giving up: {attempts} DB POST attempts failed')
 
         status = 'succeeded' if post_successful else 'failed'
         print(f'POST {status}')
@@ -542,7 +558,7 @@ def check_scanner_image_file(filepath):
 
     --------------------------------------------------------------------------
     args
-        val : string
+        filepath : string
             filename, e.g. '23020703000088001_vtile_circuit_in.png'
     --------------------------------------------------------------------------
     returns
@@ -575,6 +591,94 @@ def check_scanner_image_file(filepath):
     return filepath, qrcode, component, side
 
 
+def check_image_type(image):
+    """
+    Supported filetypes are PNG and TIFF.
+
+    --------------------------------------------------------------------------
+    args
+        image : PIL.Image instance
+    --------------------------------------------------------------------------
+    returns : bool
+        True if the image and extension match, and the actual image format is
+        supported.
+    --------------------------------------------------------------------------
+    """
+    basename = os.path.basename(image.filename)
+    extension_upper = os.path.splitext(basename)[-1][1:].upper()
+    imgfmt = image.format
+
+    extension_format_mismatch = image.format[:3] not in extension_upper[:3]
+    if extension_format_mismatch:
+        print(
+            f'WARNING ({basename}): extension does not match '
+            f'actual image format ({imgfmt})'
+        )
+
+    supform = {'PNG', 'TIFF'}
+    unsupported_format = imgfmt not in supform
+    if unsupported_format:
+        print(f'Please use {", ".join(supform)}')
+
+    # For the moment, just issue the warning, allow users to adjust
+    # if extension_format_mismatch or unsupported_format:
+    #     return False
+
+    return True
+
+
+def image_dimensions_within_bounds(filepath):
+    """
+    Is the supplied image small enough for PIL to process without complaint?
+
+    This function should be called early, so huge files are eliminated from
+    the set of potential uploads before an ssh connection is established with
+    the remote RHUL filestore.
+
+    Image.open is lazy: it opens the file but doesn't read the data into
+    memory, but it will trigger the decompression bomb exception if the image
+    has a total number of pixels above the bound (Image.MAX_IMAGE_PIXELS * 2).
+    We set Image.MAX_IMAGE_PIXELS above the default in main().
+
+    https://pillow.readthedocs.io/en/stable/reference/Image.html
+
+    This function also ensures that there is no mismatch between file
+    extension and actual image type.
+
+    --------------------------------------------------------------------------
+    args
+        filepath : string
+            filename, e.g. '23020703000088001_vtile_circuit_in.png'
+    --------------------------------------------------------------------------
+    returns : bool
+        True if file is small enough to process, False if image is huge.
+    --------------------------------------------------------------------------
+    """
+    try:
+        img = Image.open(filepath)
+    except Image.DecompressionBombError as err:
+        print(
+            f'{'-' * 80}\n'
+            f'{err}\n{filepath} omitted\n'
+            f'{'-' * 80}'
+        )
+        return False
+    except PIL.UnidentifiedImageError as err:
+        # Catch non-images with image-like file extensions or unsupported
+        # image formats with incorrect but allowed file extensions.
+        print(err)
+
+    # Catch mismatches between file extension and image type, e.g.
+    # JPEG files saved with TIFF or PNG extensions.
+    valid_image = check_image_type(img)
+
+    img.close()
+    if valid_image:
+        return True
+
+    return False
+
+
 def read_credentials(filename):
     """
     Read credentials dictionary from a file from the location specified on the
@@ -1052,7 +1156,7 @@ def limit_scans_for_upload(dbi, institute_id, image_files):
     print(f'{"#"*80}\n')
 
     # give the user a chance to review information and/or terminate the script
-    time.sleep(6)
+    time.sleep(8)
 
 
 def qrcodes_from_filenames(image_files):
@@ -1146,6 +1250,12 @@ def main():
     database entries. Tested for vtile, but should also work for vpcb_asic,
     vmotherboard and vpdu.
     """
+    # avoid warning messages from missing fields
+    logging.getLogger().setLevel(logging.ERROR)
+
+    # Handle large images from Manchester (14000 x 14000)
+    Image.MAX_IMAGE_PIXELS = 196000000
+
     args = check_arguments()
 
     image_files = sorted(
@@ -1155,8 +1265,15 @@ def main():
         )
     )
 
+    # Avoid time-consuming image format conversions if the user has already
+    # converted a TIFF to a PNG, and both TIFF and PNG files coexist.
     prefer_png_to_tiff(image_files)
 
+    # prevent users uploading unsuitable files that were scanned incorrectly
+    image_files = [
+        f for f in image_files if image_dimensions_within_bounds(f)
+    ]
+
     status = types.SimpleNamespace(success=0, unreserved_error_code=3)
     if not image_files:
         return status.unreserved_error_code
@@ -1183,7 +1300,9 @@ def main():
 
     # ------------------------------------------------------------------------
 
+    broken = collections.defaultdict(set)
     failure = []
+    scan_count = 0
     with Connect(args) as ssh:
         if not ssh:
             image_files.clear()
@@ -1213,13 +1332,33 @@ def main():
 
             if comp.generate_table_for_db_post():
                 comp.upload_image()
+                if comp.optical_inspection is not None:
+                    scan_count += 1
+                else:
+                    broken[file].add('filestore')
+
                 if args.nodb:
                     print('database POST skipped')
                     failure.append(False)
                 else:
-                    failure.append(comp.post_table())
+                    post_failed = comp.post_table()
+                    failure.append(post_failed)
+                    if post_failed:
+                        broken[file].add('database')
             else:
                 print(f'Cannot upload {file}')
+                broken[file].add('filestore')
+
+    print('\n')
+    print('-' * 80)
+    print(f'    scans uploaded: {scan_count:>5}')
+    print(f'db records created: {collections.Counter(failure)[False]:>5}')
+    print('-' * 80)
+    if broken:
+        print('files with filestore/database issues:')
+        for filename, reasons in broken.items():
+            print(f'{filename}: {", ".join(sorted(reasons))}')
+        print('-' * 80)
 
     # ------------------------------------------------------------------------
 
@@ -1230,5 +1369,5 @@ def main():
 
 
 ##############################################################################
-if __name__ == "__main__":
+if __name__ == '__main__':
     sys.exit(main())
