@@ -139,12 +139,14 @@ def check_arguments(dbi):
             "stored", object in storage;\
             "integrated", object installed (vTile on vPDU, vPDU in the detector);\
             "damaged", object damaged so unusable, waiting for a possible repair;\
+            "reworkable", object that could be repaired, usually replacing one or more SiPMs;\
             "reworked", object repaired so fully usable;\
-            "scrapped", object removed from production, never to be integrated;'
+            "scrapped", object removed from production, never to be integrated;\
+            "development", object used for R&D, e.g. DCR tests'
         ),
         choices=[
             'received', 'unbagged', 'tested', 'bagged', 'shipped',
-            'produced', 'stored', 'integrated', 'damaged', 'reworked','scrapped'
+            'produced', 'stored', 'integrated', 'damaged', 'reworkable', 'reworked', 'scrapped', 'development'
         ],
         type=str
     )
@@ -288,8 +290,8 @@ class Object:
             try:
                 tmp_id = self.dbi.get(
                     test_object, **{key: parameter}
-                ).data[select][0]
-            except KeyError:
+                ).data[select].iloc[-1]
+            except (KeyError, IndexError):
                 break
             else:
                 if not tmp_id:
