@@ -116,10 +116,14 @@ class Object:
             'integrated',
             # object damaged so unusable, waiting for a possible repair
             'damaged',
+            # object that could be repaired, usually replacing one or more SiPMs
+            'reworkable',
             # object repaired so fully usable
             'reworked',
             # object removed from production, never to be integrated
             'scrapped',
+            # object used for R&D, e.g. DCR tests
+            'development',
         ]
         self.status_var = tk.StringVar(self.root)
         self.status_dropdown = ttk.Combobox(
@@ -580,5 +584,5 @@ def main():
 
 
 ##############################################################################
-if __name__ == "__main__":
+if __name__ == '__main__':
     main()
